@@ -3,7 +3,7 @@
 
 Usage:
     exo [options] spec <cik> <spec-yaml> [--ids=<id1,id2,idN>] [--portal|--domain] [-f]
-    exo [options] spec <cik> --generate=<filename> [--scripts=<dir>]
+    exo [options] spec <cik> --generate=<filename> [--scripts=<dir>] [--asrid]
     exo [options] spec --example
 
 The --generate form creates spec YAML and scripts from a CIK.
@@ -12,6 +12,7 @@ Command options:
     --update-scripts  Update any scripts that do not match what's
                       on the filesystem
     --create          Create any resources that do not exist
+    --asrid           When generating a spec, do not convert RIDs into aliases.
     --ids substitutes values for <% id %> when matching alias
     --example         Show an annotated example spec YAML file
     --portal          Will apply the spec command to all devices in a portal
@@ -49,7 +50,7 @@ class Plugin():
         return 'spec'
     def run(self, cmd, args, options):
         if args['--example']:
-            print('''
+            s = '''
 # Example client specification file
 # Specification files are in YAML format (a superset of JSON
 # with more readable syntax and support for comments) and
@@ -92,6 +93,12 @@ dataports:
                    "properties": {"name": {"type": "string"}},
                    "required": ["name"]}
       initial: '{"name":"John Doe"}'
+    - alias: place
+      # An description of the dataport.
+      description: 'This is a place I have been'
+      # Dataport are private by default,
+      # but if you want to share one with the world
+      public: true
 
     # any dataports not listed but found in the client
     # are ignored. The spec command does not delete things.
@@ -100,9 +107,9 @@ dataports:
 scripts:
     # by default, scripts are datarules with
     # names and aliases set to the file name
-    - file: files/helloworld.lua
+    - file: test/files/helloworld.lua
     # you can also set them explicitly
-    - file: files/convert.lua
+    - file: test/files/convert.lua
       # if <% id %> is embedded in aliases
       # or script content, the --ids parameter must
       # be passed in. The spec command then expects
@@ -118,14 +125,18 @@ scripts:
       # written to each script datarule.
       #
       alias: convert<% id %>.lua
-''')
+'''
+            if not six.PY3:
+                s = s.encode('utf-8')
+            print(s)
             return
 
 
-
         input_cik = options['cik']
         rpc = options['rpc']
         ExoException = options['exception']
+        asrid = args['--asrid']
+
         if cmd == 'spec':
 
             if args['--generate'] is not None:
@@ -145,6 +156,7 @@ scripts:
                                               'aliases': True}],
                      ['listing', ['dataport', 'datarule', 'dispatch'], {}]])
                 rids = listing['dataport'] + listing['datarule']
+
                 if len(rids) > 0:
                     child_info = rpc._exomult(input_cik, [['info', rid, {'basic': True, 'description': True}] for rid in rids])
                     for idx, rid in enumerate(rids):
@@ -159,7 +171,37 @@ scripts:
                         if typ == 'dataport':
                             dp = {'name': myinfo['description']['name'],
                                   'alias': info['aliases'][rid][0],
-                                  'format': myinfo['description']['format']}
+                                  'format': myinfo['description']['format']
+                                  }
+
+                            preprocess = myinfo['description']['preprocess']
+                            if preprocess is not None and len(preprocess) > 0:
+                                def toAlias(pair):
+                                    if not asrid and pair[1] in info['aliases']:
+                                        return [pair[0], info['aliases'][pair[1]][0]]
+                                    else:
+                                        return pair
+                                dp['preprocess'] = [toAlias(x) for x in preprocess]
+
+
+                            subscribe = myinfo['description']['subscribe']
+                            if subscribe is not None and subscribe is not "":
+                                if not asrid and subscribe in info['aliases']:
+                                    dp['subscribe'] = info['aliases'][subscribe][0]
+                                else:
+                                    dp['subscribe'] = subscribe
+
+                            retention = myinfo['description']['retention']
+                            if retention is not None:
+                                count = retention['count']
+                                duration = retention['duration']
+                                if count is not None and duration is not None:
+                                    if count == 'infinity':
+                                        del retention['count']
+                                    if duration == 'infinity':
+                                        del retention['duration']
+                                    if len(retention) > 0:
+                                        dp['retention'] = retention
 
                             meta_string = myinfo['description']['meta']
                             try:
@@ -167,10 +209,18 @@ scripts:
                                 unit = meta['datasource']['unit']
                                 if len(unit) > 0:
                                     dp['unit'] = unit
+                                desc = meta['datasource']['description']
+                                if len(desc) > 0:
+                                    dp['description'] = desc
                             except:
                                 # assume unit is not present in metadata
                                 pass
                             spec.setdefault('dataports', []).append(dp)
+                            
+                            public = myinfo['description']['public']
+                            if public is not None and public:
+                                dp['public'] = public
+
 
                         elif typ == 'datarule':
                             desc = myinfo['description']
@@ -356,8 +406,19 @@ scripts:
             # for each device in our list of ciks
             for cik in ciks:
                 try:
+                    aliases = {}
                     print("Running spec on: " + cik)
                     #   apply spec [--create]
+
+                    # Get map of aliases
+                    info = rpc.info(cik, {'alias': ''}, {'aliases': True})
+                    try:
+                        for rid, alist in info['aliases'].items():
+                            for alias in alist:
+                                aliases[alias] = rid
+                    except:
+                        pass
+
                     for typ in ['dataport', 'client', 'script']:
                         if typ + 's' in spec:
                             for res in spec[typ + 's']:
@@ -407,6 +468,7 @@ scripts:
                                             rid = rpc.create_dataport(cik, format, name=name)
                                             rpc.map(cik, rid, alias)
                                             info, val = infoval(cik, alias)
+                                            aliases[alias] = rid
 
                                         # check type
                                         if info['basic']['type'] != typ:
@@ -460,39 +522,111 @@ scripts:
                                                 'Invalid spec for {0}. Unrecognized format content {1}'.format(alias, format_content))
 
                                         # check unit
-                                        if 'unit' in res:
+                                        if 'unit' in res or 'description' in res:
                                             meta_string = info['description']['meta']
                                             try:
                                                 meta = json.loads(meta_string)
                                             except:
                                                 meta = None
 
+                                            def bad_desc_msg(s):
+                                                sys.stdout.write('spec expects description for {0} to be {1}, but they are not.'.format(alias, res['description']))
                                             def bad_unit_msg(s):
                                                 sys.stdout.write('spec expects unit for {0} to be {1}, but they are not.'.format(alias, res['unit']))
-                                            def update_meta(meta):
+
+                                            if create:
+                                                if meta is None:
+                                                    meta = {'datasource':{'description':'','unit':''}}
+                                                if 'datasource' not in meta:
+                                                    meta['datasource'] = {'description':'','unit':''}
+                                                if 'unit' in res:
+                                                    meta['datasource']['unit'] = res['unit']
+                                                if 'description:' in res:
+                                                    meta['datasource']['description'] = res['description']
+
                                                 new_desc = info['description'].copy()
                                                 new_desc['meta'] = json.dumps(meta)
                                                 rpc.update(cik, {'alias': alias}, new_desc)
-                                                print('unit value for {0} updated to {1}'.format(alias, meta['datasource']['unit']))
 
-                                            if meta is None:
-                                                if create:
-                                                    update_meta({'datasource':{'description':'','unit':res['unit']}})
-                                                else:
-                                                    bad_unit_msg(', but found has no metadata at all. Pass --create to write metadata with unit.')
-                                            elif 'datasource' not in meta or 'unit' not in meta['datasource']:
-                                                if create:
-                                                    meta.setdefault('datasource', {})
-                                                    meta['datasource']['unit'] = res['unit']
-                                                    update_meta(meta)
-                                                else:
+                                            else:
+                                                if meta is None:
+                                                    sys.stdout.write('spec expects metadata but found has no metadata at all. Pass --create to write metadata.')
+                                                elif 'datasource' not in meta:
+                                                    sys.stdout.write('spec expects datasource in metadata but found its not there. Pass --create to write metadata.')
+                                                elif 'unit' not in meta['datasource']:
                                                     bad_unit_msg(', but no unit is specified in metadata. Pass --create to set unit.')
-                                            elif meta['datasource']['unit'] != res['unit']:
-                                                if create:
-                                                    meta['datasource']['unit'] = res['unit']
-                                                    update_meta(meta)
-                                                else:
+                                                elif 'description' not in meta['datasource']:
+                                                    bad_desc_msg(', but no description is specified in metadata. Pass --create to set description.')
+                                                elif 'unit' in res and meta['datasource']['unit'] != res['unit']:
                                                     bad_unit_msg(', but metadata specifies unit of {0}. Pass --create to update unit.'.format(meta['datasource']['unit']))
+                                                elif 'description' in res and meta['datasource']['description'] != res['description']:
+                                                    bad_desc_msg(', but metadata specifies description of {0}. Pass --create to update description.'.format(meta['datasource']['description']))
+
+
+                                        if 'public' in res:
+                                            resPub = res['public']
+                                            public = info['description']['public']
+                                            if public is None:
+                                                if create:
+                                                    new_desc = info['description'].copy()
+                                                    new_desc['public'] = respub
+                                                    rpc.update(cik, {'alias': alias}, new_desc)
+                                                else:
+                                                    sys.stdout.write('spec expects public for {0} to be {1}, but they are not.'.format(alias, resPub))
+                                            elif public != resPub:
+                                                sys.stdout.write('spec expects public for {0} to be {1}, but they are not.'.format(alias, resPub))
+
+
+                                        if 'subscribe' in res:
+                                            # Alias *must* be local to this CIK
+                                            resSub = res['subscribe']
+                                            # Lookup alias/name if need be
+                                            if resSub in aliases:
+                                                resSub = aliases[resSub]
+                                            subscribe = info['description']['subscribe']
+                                            if subscribe is None:
+                                                if create:
+                                                    new_desc = info['description'].copy()
+                                                    new_desc['subscribe'] = resSub
+                                                    rpc.update(cik, {'alias': alias}, new_desc)
+                                                else:
+                                                    sys.stdout.write('spec expects subscribe for {0} to be {1}, but they are not.'.format(alias, resSub))
+                                            elif subscribe != resSub:
+                                                sys.stdout.write('spec expects subscribe for {0} to be {1}, but they are not.'.format(alias, resSub))
+
+                                        if 'preprocess' in res:
+                                            def fromAliases(pair):
+                                                if pair[1] in aliases:
+                                                    return [pair[0], aliases[pair[1]]]
+                                                else:
+                                                    return pair
+                                            resPrep = [fromAliases(x) for x in res['preprocess']]
+                                            preprocess = info['description']['preprocess']
+                                            if preprocess is None or len(preprocess) == 0:
+                                                if create:
+                                                    new_desc = info['description'].copy()
+                                                    new_desc['preprocess'] = resPrep
+                                                    rpc.update(cik, {'alias': alias}, new_desc)
+                                                else:
+                                                    sys.stdout.write('spec expects preprocess for {0} to be {1}, but they are not.'.format(alias, resPrep))
+                                            elif preprocess != resPrep:
+                                                sys.stdout.write('spec expects preprocess for {0} to be {1}, but they are {2}.'.format(alias, resPrep, preprocess))
+
+                                        if 'retention' in res:
+                                            resRet = {}
+                                            if 'count' in res['retention']:
+                                                resRet['count'] = res['retention']['count']
+                                            if 'duration' in res['retention']:
+                                                resRet['duration'] = res['retention']['duration']
+
+                                            retention = info['description']['retention']
+                                            if create:
+                                                new_desc = info['description'].copy()
+                                                new_desc['retention'] = resRet
+                                                rpc.update(cik, {'alias': alias}, new_desc)
+                                            elif retention != resRet:
+                                                sys.stdout.write('spec expects retention for {0} to be {1}, but they are {2}.'.format(alias, resRet, retention))
+
 
                                     elif typ == 'script':
                                         if 'file' not in res:
